/*
 * CTU Open Contest 2012
 *
 * Sample solution: Rhinoceros Beetle
 * Author: Jan Stoklasa
 */


#include <cstdlib>
#include <iostream>
#include <vector>
#include <utility>
#include <functional>
#include <algorithm>
 
using namespace std;
 
const int MAX_PLAYERS = 10;
const int NCARDS = 13;
const int NSUITS = 4;
const int TEXASHOLDEM = 7;
 
const int Card = 0;
const int Suit = 1;
 
int toCard(char c);
int toSuit(char c);
char fromCard(int c);
char fromSuit(int c);
 
const char* hand_name[9] = {"OneCard", "Pair", "TwoPairs", "Three", "Straight", "Flush", "FullHouse", "Poker", "RoyalFlush"};
enum Figure {OneCard, Pair, TwoPairs, Three, Straight, Flush, FullHouse, Poker, RoyalFlush};
 
void dump(vector<pair<int,int> > table, pair<int,int> figure[5])
{
	for (int i=0;i<TEXASHOLDEM;i++)
	{
		cerr << fromCard(table[i].first) << fromSuit(table[i].second) << " ";
	}
	cerr << "-> ";
	for (int i=0;i<5;i++)
	{
		cerr << fromCard(figure[i].first) << fromSuit(figure[i].second) << " ";
	}
	cerr << endl;
}
 
class Hand
{
private:
 
	int cards[NCARDS];
	int suits[NSUITS];
 
public: 
	vector<pair<int,int> > deck;
	int playerNo;
	Hand(int playerNo = 0)
	{
		this->playerNo = playerNo;
		for (int card = 0; card < NCARDS; card++) 
		{
			cards[card] = 0;
		}
		for (int suit = 0; suit < NSUITS; suit++)
		{
			suits[suit] = 0;
		}
	}
 
	static int GivenCard;
	static bool gtFirstGivenCard(const pair<int, int>& x, const pair<int, int>& y)
	{
		if (y.first == GivenCard)
			return false;
		else if (x.first == GivenCard)
			return true;
		else 
			return x.first > y.first;
	}
 
	static bool gtCardThenColor(const pair<int, int>& x, const pair<int, int>& y)
	{
		if (x.first == y.first)
			return x.second > y.second;
		else
			return x.first > y.first;
	}
 
	static bool gtColorThenCard(const pair<int, int>& x, const pair<int, int>& y)
	{
		if (x.second == y.second)
			return x.first > y.first;
		else
			return x.second > y.second;
	}
 
	static bool gtAceBelow2(const pair<int, int>& x, const pair<int, int>& y)
	{	
		if (x.first == toCard('A'))
			return false;
		else if (y.first == toCard('A'))
			return true;
		else
			return x.first > y.first;
	}
 
	static bool gtColorThenCardAceBelow2(const pair<int, int>& x, const pair<int, int>& y)
	{
		if (x.second == y.second)
		{
			return gtAceBelow2(x, y);
		}
		else
			return x.second > y.second;
	}
 
	void play(char card, char suit)
	{
		cards[toCard(card)]++;
		suits[toSuit(suit)]++;
		deck.push_back(make_pair(toCard(card),toSuit(suit)));
	}
 
	bool getRoyalFlush(pair<int,int> figure[5])
	{
		sort(deck.begin(), deck.end(), gtColorThenCard);
		bool b = getRoyalFlushHelper(figure);
		if (b) 
			return true;
		else
		{
			sort(deck.begin(), deck.end(), gtColorThenCardAceBelow2);
			return getRoyalFlushHelper(figure);
		}
	}
 
	bool getRoyalFlushHelper(pair<int,int> figure[5])
	{
		int len = 1;
		int start = 0;
		for (int i = 1; i < deck.size(); i++)
		{
			if (len == 5)
			{
				break;
			}
			if (deck[i].second == deck[i-1].second &&
				(deck[i].first == deck[i-1].first - 1 || 
				deck[i].first == toCard('A') && deck[i-1].first == toCard('2')))
			{
				len++;
			}
			else
			{
				len = 1;
				start = i;
			}
		}
		if (len == 5)
		{
			for (int i = 0; i < 5; i++)
				figure[i] = deck[start + i];
			return true;
		}
		return false;
	}
 
 
	bool getN(int N, pair<int,int> figure[5])
	{
		vector<int> tuples;
		for (int card = 0; card < NCARDS; card++)
		{
			if (cards[card] == N)
			{
				tuples.push_back(card);
			}
		}
		if (tuples.size()<1)
			return false;
		sort(tuples.begin(), tuples.end(), std::greater<int>());
		GivenCard = tuples[0];
		sort(deck.begin(), deck.end(), gtFirstGivenCard);
		for (int i = 0; i < 5; i++)
			figure[i] = deck[i];
		return true;
	}
 
	bool getPoker(pair<int,int> figure[5])
	{
		return getN(4, figure);
	}
 
	bool getFullHouse(pair<int,int> figure[5])
	{
		vector<int> three;
		for (int card = 0; card < NCARDS; card++)
		{
			if (cards[card] == 3)
			{
				three.push_back(card);
			}
		}
		vector<int> two;
		for (int card = 0; card < NCARDS; card++)
		{
			if (cards[card] == 2)
			{
				two.push_back(card);
			}
		}
		if (three.size() < 1 || (three.size() == 1 && two.size() < 1))
			return false;
		sort(three.begin(), three.end(), std::greater<int>());
		sort(two.begin(), two.end(), std::greater<int>());
 
		GivenCard = three[0];
		sort(deck.begin(), deck.end(), gtFirstGivenCard);
		GivenCard = (three.size()>1) ? three[1] : two[0];
		sort(deck.begin()+3, deck.end(), gtFirstGivenCard);
		for (int i = 0; i < 5; i++)
			figure[i] = deck[i];
		return true;
	}
 
	bool getFlush(pair<int,int> figure[5])
	{
		for (int suit = 0; suit < NSUITS; suit++)
		{
			if (suits[suit] >= 5)
			{
				sort(deck.begin(), deck.end(), gtColorThenCard);
				for (int i = 0; i < TEXASHOLDEM; i++)
				{
					if (deck[i].second == suit)
					{
						for (int j = 0; j < 5; j++)
							figure[j] = deck[i+j];
						return true;
					}
				}
			}
		}
		return false;
	}
 
	bool getStraight(pair<int,int> figure[5])
	{
		sort(deck.begin(), deck.end(), gtCardThenColor);
		bool b = getStraightHelper(figure);
		if (b)
			return true;
		else
		{
			sort(deck.begin(), deck.end(), gtAceBelow2);
			return getStraightHelper(figure);
		}
	}
 
	bool getStraightHelper(pair<int,int> figure[5])
	{
		vector<pair<int,int> > straight;
		straight.push_back(deck[0]);
		for (int i = 1; i < deck.size(); i++)
		{
			if (deck[i].first != straight[straight.size()-1].first)
				straight.push_back(deck[i]);
		}
		int len = 1;
		int start = 0;
		for (int i = 1; i < straight.size(); i++)
		{
			if (len == 5)
				break;
			if (straight[i].first == straight[i-1].first - 1 || 
				(straight[i].first == toCard('A') && straight[i-1].first == toCard('2')))
			{
				len++;
			}
			else
			{
				len = 1;
				start = i;
			}
		}
		if (len == 5)
		{
			for (int i = 0; i < 5; i++)
				figure[i] = straight[start + i];
			return true;
		}
		return false;
	}
 
	bool getThree(pair<int,int> figure[5])
	{
		return getN(3, figure);
	}
 
	bool getTwoPairs(pair<int,int> figure[5])
	{
		vector<int> pairs;
		for (int card = 0; card < NCARDS; card++)
		{
			if (cards[card] == 2)
			{
				pairs.push_back(card);
			}
		}
		if (pairs.size()<2)
			return false;
		sort(pairs.begin(), pairs.end(), std::greater<int>());
		GivenCard = pairs[0];
		sort(deck.begin(), deck.end(), gtFirstGivenCard);
		GivenCard = pairs[1];
		sort(deck.begin()+2, deck.end(), gtFirstGivenCard);
		for (int i = 0; i < 5; i++)
			figure[i] = deck[i];
		return true;
	}
 
	bool getPair(pair<int,int> figure[5])
	{
		return getN(2, figure);
	}
 
	bool getOneCard(pair<int,int> figure[5])
	{
		sort(deck.begin(), deck.end(), gtCardThenColor);
		for (int i = 0; i < 5; i++)
			figure[i] = deck[i];
		return true;
	}
 
	Figure classify(pair<int, int> figure[5])
	{
		if (getRoyalFlush(figure))
			return RoyalFlush;
		else if (getPoker(figure))
			return Poker;
		else if (getFullHouse(figure))
			return FullHouse;
		else if (getFlush(figure))
			return Flush;
		else if (getStraight(figure))
			return Straight;
		else if (getThree(figure))
			return Three;
		else if (getTwoPairs(figure))
			return TwoPairs;
		else if (getPair(figure))
			return Pair;
		else if (getOneCard(figure))
			return OneCard;
		else
			printf("rhino: internal error when classifying hand - shouldn't happen");
	}
 
	static bool greaterThan(const Hand& lhs, const Hand& rhs)
	{
		pair<int, int> lhfigure[5];
		pair<int, int> rhfigure[5];
		Figure lhf = ((Hand&)lhs).classify(lhfigure);
		Figure rhf = ((Hand&)rhs).classify(rhfigure);
 
		if (lhf > rhf)
		{
			return true;
		}
		else if (rhf > lhf)
		{
			return false;
		}
		else // lhf == rhf
		{
			for (int i = 0; i < 5; i++)
			{
				if (lhfigure[i].first > rhfigure[i].first)
					return true;
				else if (lhfigure[i].first < rhfigure[i].first)
					return false;
			}
			return false; 
		}
	}
 
	static bool byPlayerNumbers(const Hand& lhs, const Hand& rhs)
	{
		return lhs.playerNo < rhs.playerNo;
	}
};
 
int Hand::GivenCard = 0; 
 
char table[5][2];
char player[MAX_PLAYERS][2][2];
Hand hands[MAX_PLAYERS];
int caseNo = 1;
int main(void)
{
  int P; 
  while (cin >> P >> table[0][Card] >> table[0][Suit] >> table[1][Card] >> table[1][Suit] >> table[2][Card] >> table[2][Suit] >> table[3][Card] >> table[3][Suit] >> table[4][Card] >> table[4][Suit])
  {
	Hand common;
	for (int i = 0; i < 5; i++) 
	{
		common.play(table[i][Card],table[i][Suit]);
	}
	for (int p = 0; p < P ; p++)
	{
		hands[p] = common;
		hands[p].playerNo = p+1;
		for (int i = 0; i < 2; i++)
		{
			cin >> player[p][i][Card] >> player[p][i][Suit];
			hands[p].play(player[p][i][Card],player[p][i][Suit]);
		}
		pair <int, int> figure[5];
		Figure f = hands[p].classify(figure);
		//cerr << hand_name[f] << ": ";dump(hands[p].deck, figure);
	}
	sort(hands, hands+P, Hand::greaterThan);
	//cout << "--" << caseNo++ << endl;
	Hand& winner = hands[0]; 
	for (int i = 0; i < P; i++)
	{
		if (!Hand::greaterThan(winner,hands[i]))
		{
			if (i > 0) 
				cout << " ";
			cout << hands[i].playerNo;
		}
		else
			break;
	}
	cout << endl;
  }
}
 
int toCard(char c)
{
    switch (c) {
      case '2': return 0;
      case '3': return 1;
      case '4': return 2;
      case '5': return 3;
	  case '6': return 4;
      case '7': return 5;
      case '8': return 6;
      case '9': return 7;
      case 'X': return 8;
      case 'J': return 9;
      case 'Q': return 10;
      case 'K': return 11;
      case 'A': return 12;
	  default: printf("rhino: invalid card in input");
    }
}
 
int toSuit(char c)
{
	switch (c) 
	{
      case 'c': return 0;
      case 'd': return 1;
      case 'h': return 2;
      case 's': return 3;
	  default: printf("rhino: invalid suit in input");
    }
}
 
char fromCard(int c)
{
    switch (c) {
      case 0: return '2';
      case 1: return '3';
      case 2: return '4';
      case 3: return '5';
	  case 4: return '6';
      case 5: return '7';
      case 6: return '8';
      case 7: return '9';
      case 8: return 'X';
      case 9: return 'J';
      case 10: return 'Q';
      case 11: return 'K';
      case 12: return 'A';
	  default: printf("rhino: invalid card in input");
    }
}
 
char fromSuit(int c)
{
	switch (c) 
	{
      case 0: return 'c';
      case 1: return 'd';
      case 2: return 'h';
      case 3: return 's';
	  default: printf("rhino: invalid suit in input");
    }
}

