/*
 * CTU Open Contest 2012
 *
 * Sample solution: Rhinoceros Beetle
 * Author: Martin Kacer
 */

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.util.Arrays;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.StringTokenizer;

@SuppressWarnings("boxing")
public class rhino {
	StringTokenizer st = new StringTokenizer("");
	BufferedReader input = new BufferedReader(new InputStreamReader(System.in));
	void useLine(String s) { st = new StringTokenizer(s); }
	void useNextLine() throws Exception { useLine(input.readLine()); }
	String nextToken() throws Exception {
		while (!st.hasMoreTokens()) st = new StringTokenizer(input.readLine());
		return st.nextToken();
	}
	int nextInt() throws Exception {
		return Integer.parseInt(nextToken());
	}
	public static void main(String[] args) throws Exception {
		rhino instance = new rhino();
		while (instance.run()) { /*repeat*/ }
	}
	
	static final int MAX_PLAYERS = 10;
	
	static enum Category {HIGH, PAIR, TWOPAIR, THREE, STRAIGHT, FLUSH, FULL, FOUR, STRFLUSH}
	static final String RANKS = "23456789XJQKA";
	static final char[] RANKS_CH = ("A" + RANKS).toCharArray();
	
	static final int[][] COMB_5_OF_7 = new int[][] {
		{0,1,2,3,4}, {0,1,2,3,5}, {0,1,2,3,6}, {0,1,2,4,5}, {0,1,2,4,6}, {0,1,2,5,6}, {0,1,3,4,5},
		{0,1,3,4,6}, {0,1,3,5,6}, {0,1,4,5,6}, {0,2,3,4,5}, {0,2,3,4,6}, {0,2,3,5,6}, {0,2,4,5,6},
		{0,3,4,5,6}, {1,2,3,4,5}, {1,2,3,4,6}, {1,2,3,5,6}, {1,2,4,5,6}, {1,3,4,5,6}, {2,3,4,5,6}
	};

	static class RankComparator implements Comparator<String> {
		final String privileged;
		/** @param privileged ranks with higher importance, in increasing priority */
		RankComparator(String privileged) {
			this.privileged = privileged;
		}
		public int compare(String h1, String h2) {
			int diff = privileged.indexOf(h2.charAt(0)) - privileged.indexOf(h1.charAt(0));
			return (diff != 0) ? diff : RANKS.indexOf(h2.charAt(0)) - RANKS.indexOf(h1.charAt(0));
		}
		static final RankComparator INSTANCE = new RankComparator("");
	}
	
	void dump(String[] h) {
		for (int i = 0; i < 5; ++i) System.err.print(h[i] + " ");
		System.err.println();
	}

	Set<Character> usedRanks = new HashSet<Character>();
	Map<Character,Integer> multiRanks = new HashMap<Character, Integer>();
	Set<Character> usedSuits = new HashSet<Character>();
	/** Assigns a category. Sorts to have the most important card first. */
	Category categorize(String[] hand) {
		usedRanks.clear(); usedSuits.clear(); multiRanks.clear();
		for (String c : hand) {
			char r = c.charAt(0);
			if (!usedRanks.add(r)) {
				Integer m = multiRanks.get(r);
				multiRanks.put(r, m == null ? 2 : m+1);
			}
			usedSuits.add(c.charAt(1));
		}
		boolean str = false;
		int cnt = 0;
		for (char ch : RANKS_CH) {
			if (usedRanks.contains(ch)) ++cnt; else cnt = 0;
			if (cnt == 5) { str = true; break; }
		}
		Category cat;
		Comparator<String> comp = RankComparator.INSTANCE;
		if (str && usedSuits.size() == 1) {
			cat = Category.STRFLUSH;
			comp = new RankComparator(RANKS.substring(0, RANKS.length()-1));
		} else if (multiRanks.values().contains(4)) {
			cat = Category.FOUR;
			comp = new RankComparator("" + multiRanks.keySet().iterator().next());
		} else if (multiRanks.values().contains(2) && multiRanks.values().contains(3)) {
			cat = Category.FULL;
			char ch3 = '?', ch2 = '?';
			for (Map.Entry<Character,Integer> e : multiRanks.entrySet()) {
				if (e.getValue() == 3) ch3 = e.getKey(); else ch2 = e.getKey();
			}
			comp = new RankComparator("" + ch2 + ch3);
		} else if (usedSuits.size() == 1) {
			cat = Category.FLUSH;
		} else if (str) {
			cat = Category.STRAIGHT;
			comp = new RankComparator(RANKS.substring(0, RANKS.length()-1));
		} else if (multiRanks.values().contains(3)) {
			cat = Category.THREE;
			comp = new RankComparator("" + multiRanks.keySet().iterator().next());
		} else if (multiRanks.size() == 2) {
			cat = Category.TWOPAIR;
			Iterator<Character> it = multiRanks.keySet().iterator();
			char ch1 = it.next(), ch2 = it.next();
			comp = new RankComparator((RANKS.indexOf(ch1) < RANKS.indexOf(ch2))
					? "" + ch1 + ch2 : "" + ch2 + ch1);
		} else if (!multiRanks.isEmpty()) {
			cat = Category.PAIR;
			comp = new RankComparator("" + multiRanks.keySet().iterator().next());
		} else
			cat = Category.HIGH;
		Arrays.sort(hand, comp);
		return cat;
	}
	
	int compare(String[] hand1, String[] hand2) {
		if (hand1 == null) return -1;
		String[] h1 = Arrays.copyOf(hand1, 5), h2 = Arrays.copyOf(hand2, 5);
		Category c1 = categorize(h1);
		Category c2 = categorize(h2);
		if (c1 != c2) return c1.ordinal() - c2.ordinal();
		for (int i = 0; i < 5; ++i) {
			int idx1 = RANKS.indexOf(h1[i].charAt(0));
			int idx2 = RANKS.indexOf(h2[i].charAt(0));
			if (idx1 != idx2) return idx1 - idx2;
		}
		return 0;
	}
	
	String table[] = new String[5];
	int[] answer = new int[MAX_PLAYERS];
	
	String[] getBestForPlayer(String hole1, String hole2) {
		String[] best = null;
		String[] avail = Arrays.copyOf(table, 7), act = new String[5];
		avail[5] = hole1; avail[6] = hole2;
		for (int i = 0; i < COMB_5_OF_7.length; ++i) {
			for (int j = 0; j < 5; ++j) act[j] = avail[COMB_5_OF_7[i][j]];
			if (compare(best, act) < 0) {
				best = Arrays.copyOf(act, 5);
			}
		}
		return best;
	}

	boolean run() throws Exception {
		String s = input.readLine();
		if (s == null) return false;
		useLine(s);
		int players = nextInt();
		for (int i = 0; i < 5; ++i) {
			table[i] = nextToken();
		}
		int anslen = 0;
		String[] best = null;
		for (int p = 1; p <= players; ++p) {
			String[] cur = getBestForPlayer(nextToken(), nextToken());
			//dump(cur);
			int cmp = compare(best, cur);
			if (cmp < 0) {
				best = cur;
				anslen = 1;
				answer[0] = p;
			} else if (cmp == 0) {
				answer[anslen++] = p;
			}
		}
		for (int i = 0; i < anslen; ++i) {
			if (i > 0) System.out.print(' ');
			System.out.print(answer[i]);
		}
		System.out.println();
		return true;
	}
}
