#include <stdio.h>
#include <string.h>
#include <vector>

using namespace std;

#define NMAX 111
#define QMOD 65535

vector<int> vec[NMAX * NMAX];
char A[NMAX][NMAX];
int H[NMAX][NMAX], V[NMAX][NMAX];
int M, N, P, W, i, j, k, node, nh, nv, li, ls, maxflow;
int q[QMOD + 1], p[2 * NMAX * NMAX];
int flow[2 * NMAX * NMAX][NMAX], pflow[2 * NMAX * NMAX], flow0[2 * NMAX * NMAX];
char t[2 * NMAX * NMAX], inq[2 * NMAX * NMAX], marked[2 * NMAX * NMAX];

int ComputeMaximumFlow() {
	memset(flow, 0, sizeof(flow));
	memset(flow0, 0, sizeof(flow0));
	memset(pflow, 0, sizeof(pflow));
	memset(p, 0, sizeof(p));

	// Do the flow.
	maxflow = 0;
	
	while (1) {
		//fprintf(stderr, "maxflow=%d\n", maxflow);

		li = 0;
		ls = 0;
	
		for (i = 1; i <= nh; i++) {
			if (!flow0[i]) {
				inq[i] = 1;
				q[ls] = i;
				ls = (ls + 1) & QMOD;
				t[i] = 1;
			} else {
				inq[i] = 0;
			}
			p[i] = 0;
		}

		for (i = nh + 1; i <= nh + nv; i++)
			inq[i] = 0;
		
		while (li != ls) {
			i = q[li];
			li = (li + 1) & QMOD;

			if (i <= nh) {
				for (k = 0; k < vec[i].size(); k++) {
					j = vec[i][k];
					if (!flow[i][k] && !inq[j]) {
						p[j] = i;
						t[j] = 1;
						inq[j] = 1;
						q[ls] = j;
						ls = (ls + 1) & QMOD;
					}
				}
			} else {
				if (pflow[i] > 0 && !inq[pflow[i]]) {
					p[pflow[i]] = i;
					t[pflow[i]] = 2;
					inq[pflow[i]] = 1;
					q[ls] = pflow[i];
					ls = (ls + 1) & QMOD;
				}
			}	
		}

		memset(marked, 0, sizeof(marked));
		k = 0;
		for (i = nh + 1; i <= nh + nv; i++) {
			if (!inq[i] || pflow[i] > 0)
				continue;
			node = i;

			// Retrace the path back from node to 0.
			while (node > 0) {
				if (marked[node])
					break;
				marked[node] = 1;
				node = p[node];
			}

			if (node == 0) {
				node = i;
				maxflow++;
				k++;

				while (node > 0) {
					if (t[node] == 1) {
						if (p[node] > 0) {
							flow[p[node]][node] = 1;
							pflow[node] = p[node];
						} else
							flow0[node] = 1;
						//fprintf(stderr, "inc flow %d->%d\n", p[node], node);
					} else {
						flow[node][p[node]] = 0;
						//fprintf(stderr, "dec flow %d->%d\n", node, p[node]);
					}
					node = p[node];
				}
			}
		}

		if (!k)
			break;
	}
}

int main() {
	//freopen("x.txt", "r", stdin);
	int T;
	scanf("%d", &T);
	
	while (T--) {
		scanf("%d %d", &M, &N);
		memset(A, 0, sizeof(A));
		scanf("%d", &P);
		for (k = 1; k <= P; k++) {
			scanf("%d %d", &i, &j);
			A[i][j] = '*';
		}
		scanf("%d", &W);
		for (k = 1; k <= W; k++) {
			scanf("%d %d", &i, &j);
			A[i][j] = '#';
		}

		// Get the horizontal segments.
		memset(H, 0, sizeof(H));
		nh = 0;
		for (i = 1; i <= M; i++)
			for (j = 1; j <= N; j++)
				if (A[i][j] == '*' && !H[i][j]) {
					nh++;
					vec[nh].clear();
					H[i][j] = nh;
					P = j;
					while (P - 1 >= 1 && A[i][P - 1] != '#') {
						P--;
						H[i][P] = nh;
					}
					P = j;
					while (P + 1 <= N && A[i][P + 1] != '#') {
						P++;
						H[i][P] = nh;
					}	
				}

		// Get the vertical segments.
		memset(V, 0, sizeof(V));
		nv = 0;
		for (i = 1; i <= M; i++)
			for (j = 1; j <= N; j++)
				if (A[i][j] == '*' && !V[i][j]) {
					nv++;
					V[i][j] = nv;
					P = i;
					while (P - 1 >= 1 && A[P - 1][j] != '#') {
						P--;
						V[P][j] = nv;
					}
					P = i;
					while (P + 1 <= M && A[P + 1][j] != '#') {
						P++;
						V[P][j] = nv;
					}
				}

		// Compute the maximum matching.		
		for (i = 1; i <= M; i++)
			for (j = 1; j <= N; j++)
				if (A[i][j] == '*')
					vec[H[i][j]].push_back(nh + V[i][j]);

		//fprintf(stderr, "nh=%d nv=%d\n", nh, nv);

		ComputeMaximumFlow();
		printf("%d\n", maxflow);
	}
	
	return 0;
}
